@@ -1,10 +1,12 @@
+use chrono::{DateTime, Utc};
 use futures::channel::mpsc;
 use irc::proto::{self, command, Command};
-use itertools::Itertools;
+use itertools::{Either, Itertools};
 use std::collections::{BTreeMap, HashMap, HashSet};
 use std::fmt;
 use std::time::{Duration, Instant};
 
+use crate::message::server_time;
 use crate::time::Posix;
 use crate::user::{Nick, NickRef};
 use crate::{config, message, mode, Buffer, Server, User};
@@ -485,12 +487,8 @@ impl Client {
                 }
 
                 // Send JOIN
-                for channel in &self.config.channels {
-                    if let Some(keys) = self.config.channel_keys.get(channel) {
-                        let _ = self.sender.try_send(command!("JOIN", channel, keys));
-                    } else {
-                        let _ = self.sender.try_send(command!("JOIN", channel));
-                    }
+                for message in group_joins(&self.config) {
+                    let _ = self.sender.try_send(message);
                 }
             }
             // QUIT
@@ -534,6 +532,15 @@ impl Client {
                     channel.users.insert(user);
                 }
             }
+            Command::KICK(channel, victim, _) => {
+                if victim == self.nickname().as_ref() {
+                    self.chanmap.remove(channel);
+                } else if let Some(channel) = self.chanmap.get_mut(channel) {
+                    channel
+                        .users
+                        .remove(&User::from(Nick::from(victim.as_str())));
+                }
+            }
             Command::Numeric(RPL_WHOREPLY, args) => {
                 let target = args.get(1)?;
 
@@ -638,9 +645,39 @@ impl Client {
                     }
                 }
             }
-            #[cfg(feature = "dev")]
-            // Suppress topic during development to prevent history spam
-            Command::Numeric(RPL_TOPIC | RPL_TOPICWHOTIME, _) => return None,
+            Command::TOPIC(channel, topic) => {
+                if let Some(channel) = self.chanmap.get_mut(channel) {
+                    channel.topic.text = topic.to_owned();
+
+                    channel.topic.who = message
+                        .user()
+                        .map(|user| user.username().unwrap().to_string());
+                    channel.topic.time = Some(server_time(&message));
+                }
+            }
+            Command::Numeric(RPL_TOPIC, args) => {
+                if let Some(channel) = self.chanmap.get_mut(&args[1]) {
+                    channel.topic.text = Some(args.get(2)?.to_owned());
+                }
+                // Exclude topic message from history to prevent spam during dev
+                #[cfg(feature = "dev")]
+                return None;
+            }
+            Command::Numeric(RPL_TOPICWHOTIME, args) => {
+                if let Some(channel) = self.chanmap.get_mut(&args[1]) {
+                    channel.topic.who = Some(args.get(2)?.to_string());
+                    channel.topic.time = Some(
+                        args.get(3)?
+                            .parse::<u64>()
+                            .ok()
+                            .map(Posix::from_seconds)?
+                            .datetime()?,
+                    );
+                }
+                // Exclude topic message from history to prevent spam during dev
+                #[cfg(feature = "dev")]
+                return None;
+            }
             _ => {}
         }
 
@@ -665,7 +702,17 @@ impl Client {
         &self.channels
     }
 
-    fn users<'a>(&'a self, channel: &str) -> &'a [User] {
+    fn topic<'a>(&'a self, channel: &str) -> Option<&'a Topic> {
+        self.chanmap.get(channel).map(|channel| &channel.topic)
+    }
+
+    fn resolve_user_attributes<'a>(&'a self, channel: &str, user: &User) -> Option<&'a User> {
+        self.chanmap
+            .get(channel)
+            .and_then(|channel| channel.users.get(user))
+    }
+
+    pub fn users<'a>(&'a self, channel: &str) -> &'a [User] {
         self.users
             .get(channel)
             .map(Vec::as_slice)
@@ -789,7 +836,7 @@ impl Map {
         }
     }
 
-    pub fn nickname(&self, server: &Server) -> Option<NickRef> {
+    pub fn nickname<'a>(&'a self, server: &Server) -> Option<NickRef<'a>> {
         self.client(server).map(Client::nickname)
     }
 
@@ -811,6 +858,16 @@ impl Map {
         }
     }
 
+    pub fn resolve_user_attributes<'a>(
+        &'a self,
+        server: &Server,
+        channel: &str,
+        user: &User,
+    ) -> Option<&'a User> {
+        self.client(server)
+            .and_then(|client| client.resolve_user_attributes(channel, user))
+    }
+
     pub fn get_channel_users<'a>(&'a self, server: &Server, channel: &str) -> &'a [User] {
         self.client(server)
             .map(|client| client.users(channel))
@@ -823,6 +880,12 @@ impl Map {
             .unwrap_or_default()
     }
 
+    pub fn get_channel_topic<'a>(&'a self, server: &Server, channel: &str) -> Option<&'a Topic> {
+        self.client(server)
+            .map(|client| client.topic(channel))
+            .unwrap_or_default()
+    }
+
     pub fn get_channels<'a>(&'a self, server: &Server) -> &'a [String] {
         self.client(server)
             .map(|client| client.channels())
@@ -950,6 +1013,14 @@ enum RegistrationStep {
 pub struct Channel {
     pub users: HashSet<User>,
     pub last_who: Option<WhoStatus>,
+    pub topic: Topic,
+}
+
+#[derive(Default, Debug, Clone)]
+pub struct Topic {
+    pub text: Option<String>,
+    pub who: Option<String>,
+    pub time: Option<DateTime<Utc>>,
 }
 
 #[derive(Debug, Clone, Copy)]
@@ -958,3 +1029,53 @@ pub enum WhoStatus {
     Receiving,
     Done(Instant),
 }
+
+/// Group channels together into as few JOIN messages as possible
+fn group_joins(config: &config::Server) -> impl Iterator<Item = proto::Message> + '_ {
+    const MAX_LEN: usize = proto::format::BYTE_LIMIT - b"JOIN \r\n".len();
+
+    let (without_keys, with_keys): (Vec<_>, Vec<_>) =
+        config.channels.iter().partition_map(|channel| {
+            config
+                .channel_keys
+                .get(channel)
+                .map(|key| Either::Right((channel, key)))
+                .unwrap_or(Either::Left(channel))
+        });
+
+    let joins_without_keys = without_keys
+        .into_iter()
+        .scan(0, |count, channel| {
+            // Channel + a comma
+            *count += channel.len() + 1;
+
+            let chunk = *count / MAX_LEN;
+
+            Some((chunk, channel))
+        })
+        .into_group_map()
+        .into_values()
+        .map(|channels| command!("JOIN", channels.into_iter().join(",")));
+
+    let joins_with_keys = with_keys
+        .into_iter()
+        .scan(0, |count, (channel, key)| {
+            // Channel + key + a comma for each
+            *count += channel.len() + key.len() + 2;
+
+            let chunk = *count / MAX_LEN;
+
+            Some((chunk, (channel, key)))
+        })
+        .into_group_map()
+        .into_values()
+        .map(|values| {
+            command!(
+                "JOIN",
+                values.iter().map(|(c, _)| c).join(","),
+                values.iter().map(|(_, k)| k).join(",")
+            )
+        });
+
+    joins_without_keys.chain(joins_with_keys)
+}
