@@ -6,11 +6,12 @@ use futures::{future, Future, FutureExt};
 use itertools::Itertools;
 use tokio::time::Instant;
 
-use crate::config::buffer::{Exclude, ServerMessages};
+use crate::config::buffer::Exclude;
 use crate::history::{self, History};
 use crate::message::{self, Limit};
 use crate::time::Posix;
-use crate::user::{Nick, NickRef};
+use crate::user::Nick;
+use crate::{config, input};
 use crate::{server, Buffer, Config, Input, Server, User};
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
@@ -166,16 +167,20 @@ impl Manager {
         }
     }
 
-    pub fn record_input(&mut self, input: Input, our_nick: NickRef) {
-        if let Some(message) = input.message(our_nick) {
+    pub fn record_input(&mut self, input: Input, user: User) {
+        if let Some(message) = input.message(user) {
             self.record_message(input.server(), message);
         }
 
         if let Some(text) = input.raw() {
-            self.data.input.push(input.buffer(), text.to_string());
+            self.data.input.record(input.buffer(), text.to_string());
         }
     }
 
+    pub fn record_draft(&mut self, draft: input::Draft) {
+        self.data.input.store_draft(draft);
+    }
+
     pub fn record_message(&mut self, server: &Server, message: crate::Message) {
         self.data.add_message(
             server.clone(),
@@ -189,13 +194,13 @@ impl Manager {
         server: &Server,
         channel: &str,
         limit: Option<Limit>,
-        server_messages: &ServerMessages,
+        buffer_config: &config::Buffer,
     ) -> Option<history::View<'_>> {
         self.data.history_view(
             server,
             &history::Kind::Channel(channel.to_string()),
             limit,
-            server_messages,
+            buffer_config,
         )
     }
 
@@ -203,10 +208,10 @@ impl Manager {
         &self,
         server: &Server,
         limit: Option<Limit>,
-        server_messages: &ServerMessages,
+        buffer_config: &config::Buffer,
     ) -> Option<history::View<'_>> {
         self.data
-            .history_view(server, &history::Kind::Server, limit, server_messages)
+            .history_view(server, &history::Kind::Server, limit, buffer_config)
     }
 
     pub fn get_query_messages(
@@ -214,13 +219,13 @@ impl Manager {
         server: &Server,
         nick: &Nick,
         limit: Option<Limit>,
-        server_messages: &ServerMessages,
+        buffer_config: &config::Buffer,
     ) -> Option<history::View<'_>> {
         self.data.history_view(
             server,
             &history::Kind::Query(nick.clone()),
             limit,
-            server_messages,
+            buffer_config,
         )
     }
 
@@ -338,7 +343,7 @@ impl Manager {
         });
     }
 
-    pub fn input_history<'a>(&'a self, buffer: &Buffer) -> &'a [String] {
+    pub fn input<'a>(&'a self, buffer: &Buffer) -> input::Cache<'a> {
         self.data.input.get(buffer)
     }
 }
@@ -364,7 +369,7 @@ fn with_limit<'a>(
 #[derive(Debug, Default)]
 struct Data {
     map: HashMap<server::Server, HashMap<history::Kind, History>>,
-    input: history::Input,
+    input: input::Storage,
 }
 
 impl Data {
@@ -427,7 +432,7 @@ impl Data {
         server: &server::Server,
         kind: &history::Kind,
         limit: Option<Limit>,
-        server_messages: &ServerMessages,
+        buffer_config: &config::Buffer,
     ) -> Option<history::View> {
         let History::Full {
             messages,
@@ -444,32 +449,34 @@ impl Data {
             .iter()
             .filter(|message| match message.target.source() {
                 message::Source::Server(Some(source)) => {
-                    let source_config = server_messages.get(source);
+                    if let Some(source_config) = buffer_config.server_messages.get(source) {
+                        match source_config.exclude {
+                            Exclude::All => false,
+                            Exclude::None => true,
+                            Exclude::Smart(seconds) => {
+                                if let Some(nick) = source.nick() {
+                                    !smart_filter_message(
+                                        message,
+                                        &seconds,
+                                        most_recent_messages.get(nick),
+                                    )
+                                } else if let Some(nickname) =
+                                    message.text.split(' ').collect::<Vec<_>>().get(1)
+                                {
+                                    let nick = Nick::from(*nickname);
 
-                    match source_config.exclude {
-                        Exclude::All => false,
-                        Exclude::None => true,
-                        Exclude::Smart(seconds) => {
-                            if let Some(nick) = source.nick() {
-                                !smart_filter_message(
-                                    message,
-                                    &seconds,
-                                    most_recent_messages.get(nick),
-                                )
-                            } else if let Some(nickname) =
-                                message.text.split(' ').collect::<Vec<_>>().get(1)
-                            {
-                                let nick = Nick::from(*nickname);
-
-                                !smart_filter_message(
-                                    message,
-                                    &seconds,
-                                    most_recent_messages.get(&nick),
-                                )
-                            } else {
-                                true
+                                    !smart_filter_message(
+                                        message,
+                                        &seconds,
+                                        most_recent_messages.get(&nick),
+                                    )
+                                } else {
+                                    true
+                                }
                             }
                         }
+                    } else {
+                        true
                     }
                 }
                 crate::message::Source::User(message_user) => {
