@@ -1,11 +1,14 @@
+use std::collections::HashMap;
+
 use chrono::Utc;
 use irc::proto;
 use irc::proto::format;
 
 use crate::time::Posix;
-use crate::user::NickRef;
 use crate::{command, message, Buffer, Command, Message, Server, User};
 
+const INPUT_HISTORY_LENGTH: usize = 100;
+
 pub fn parse(buffer: Buffer, input: &str) -> Result<Input, Error> {
     let content = match command::parse(input, Some(&buffer)) {
         Ok(command) => Content::Command(command),
@@ -52,7 +55,7 @@ impl Input {
         self.buffer.server()
     }
 
-    pub fn message(&self, our_nick: NickRef) -> Option<Message> {
+    pub fn message(&self, user: User) -> Option<Message> {
         let to_target = |target: String, source| {
             if proto::is_channel(&target) {
                 Some(message::Target::Channel {
@@ -76,10 +79,7 @@ impl Input {
                 received_at: Posix::now(),
                 server_time: Utc::now(),
                 direction: message::Direction::Sent,
-                target: to_target(
-                    target,
-                    message::Source::User(User::from(our_nick.to_owned())),
-                )?,
+                target: to_target(target, message::Source::User(user))?,
                 text,
             }),
             Command::Me(target, action) => Some(Message {
@@ -87,7 +87,7 @@ impl Input {
                 server_time: Utc::now(),
                 direction: message::Direction::Sent,
                 target: to_target(target, message::Source::Action)?,
-                text: message::action_text(our_nick, &action),
+                text: message::action_text(user.nickname(), &action),
             }),
             _ => None,
         }
@@ -126,6 +126,49 @@ impl Content {
     }
 }
 
+#[derive(Debug, Clone)]
+pub struct Draft {
+    pub buffer: Buffer,
+    pub text: String,
+}
+
+#[derive(Debug, Clone, Default)]
+pub struct Storage {
+    sent: HashMap<Buffer, Vec<String>>,
+    draft: HashMap<Buffer, String>,
+}
+
+impl Storage {
+    pub fn get<'a>(&'a self, buffer: &Buffer) -> Cache<'a> {
+        Cache {
+            history: self.sent.get(buffer).map(Vec::as_slice).unwrap_or_default(),
+            draft: self
+                .draft
+                .get(buffer)
+                .map(AsRef::as_ref)
+                .unwrap_or_default(),
+        }
+    }
+
+    pub fn record(&mut self, buffer: &Buffer, text: String) {
+        self.draft.remove(buffer);
+        let history = self.sent.entry(buffer.clone()).or_default();
+        history.insert(0, text);
+        history.truncate(INPUT_HISTORY_LENGTH);
+    }
+
+    pub fn store_draft(&mut self, draft: Draft) {
+        self.draft.insert(draft.buffer, draft.text);
+    }
+}
+
+/// Cached values for a buffers input
+#[derive(Debug, Clone, Copy)]
+pub struct Cache<'a> {
+    pub history: &'a [String],
+    pub draft: &'a str,
+}
+
 #[derive(Debug, thiserror::Error)]
 pub enum Error {
     #[error(
