@@ -155,6 +155,10 @@ impl User {
             .unwrap_or(AccessLevel::Member)
     }
 
+    pub fn has_access_level(&self, access_level: AccessLevel) -> bool {
+        self.access_levels.get(&access_level).is_some()
+    }
+
     pub fn update_access_level(&mut self, operation: mode::Operation, mode: mode::Channel) {
         if let Ok(level) = AccessLevel::try_from(mode) {
             match operation {
@@ -203,7 +207,7 @@ impl From<proto::User> for User {
 
 impl fmt::Display for User {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "{}", self.nickname())
+        write!(f, "{}{}", self.highest_access_level(), self.nickname())
     }
 }
 
@@ -216,6 +220,12 @@ impl fmt::Display for Nick {
     }
 }
 
+impl AsRef<str> for Nick {
+    fn as_ref(&self) -> &str {
+        self.0.as_ref()
+    }
+}
+
 impl From<String> for Nick {
     fn from(nick: String) -> Self {
         Nick(nick)
@@ -273,7 +283,7 @@ impl<'a> PartialEq<Nick> for NickRef<'a> {
     }
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize)]
 pub enum AccessLevel {
     Member,
     Voice,
