@@ -1,11 +1,14 @@
 use data::server::Server;
-use data::{channel, client, history, message, Config};
-use iced::widget::{column, container, row, vertical_space};
+use data::User;
+use data::{channel, history, message, Config};
+use iced::widget::{column, container, row};
 use iced::{Command, Length};
 
 use super::{input_view, scroll_view, user_context};
 use crate::theme;
-use crate::widget::{selectable_text, Collection, Element};
+use crate::widget::{selectable_text, Element};
+
+mod topic;
 
 #[derive(Debug, Clone)]
 pub enum Message {
@@ -21,7 +24,6 @@ pub enum Event {
 
 pub fn view<'a>(
     state: &'a Channel,
-    status: client::Status,
     clients: &'a data::client::Map,
     history: &'a history::Manager,
     settings: &'a channel::Settings,
@@ -29,7 +31,7 @@ pub fn view<'a>(
     is_focused: bool,
 ) -> Element<'a, Message> {
     let buffer = state.buffer();
-    let input_history = history.input_history(&buffer);
+    let input = history.input(&buffer);
     let our_nick = clients.nickname(&state.server);
 
     let messages = container(
@@ -39,38 +41,33 @@ pub fn view<'a>(
             history,
             config,
             move |message| {
-                let timestamp = config
-                    .buffer
-                    .format_timestamp(&message.server_time)
-                    .map(|timestamp| selectable_text(timestamp).style(theme::Text::Transparent));
+                let timestamp =
+                    config
+                        .buffer
+                        .format_timestamp(&message.server_time)
+                        .map(|timestamp| {
+                            selectable_text(timestamp).style(theme::selectable_text::transparent)
+                        });
 
                 match message.target.source() {
                     message::Source::User(user) => {
                         let nick = user_context::view(
                             selectable_text(config.buffer.nickname.brackets.format(user)).style(
-                                theme::Text::Nickname(
-                                    user.color_seed(&config.buffer.nickname.color),
-                                    false,
-                                ),
+                                |theme| {
+                                    theme::selectable_text::nickname(
+                                        theme,
+                                        user.color_seed(&config.buffer.nickname.color),
+                                        user.is_away(),
+                                    )
+                                },
                             ),
-                            user.clone(),
+                            user,
+                            state.buffer(),
                         )
                         .map(scroll_view::Message::UserContext);
-                        let row_style = match our_nick {
-                            Some(nick)
-                                if message::reference_user(
-                                    user.nickname(),
-                                    nick,
-                                    &message.text,
-                                ) =>
-                            {
-                                theme::Container::Highlight
-                            }
-                            _ => theme::Container::Default,
-                        };
 
                         let space = selectable_text(" ");
-                        let message = selectable_text(&message.text);
+                        let text = selectable_text(&message.text);
 
                         Some(
                             container(
@@ -78,25 +75,38 @@ pub fn view<'a>(
                                     .push_maybe(timestamp)
                                     .push(nick)
                                     .push(space)
-                                    .push(message),
+                                    .push(text),
                             )
-                            .style(row_style)
+                            .style(move |theme, status| match our_nick {
+                                Some(nick)
+                                    if message::reference_user(
+                                        user.nickname(),
+                                        nick,
+                                        &message.text,
+                                    ) =>
+                                {
+                                    theme::container::highlight(theme, status)
+                                }
+                                _ => Default::default(),
+                            })
                             .into(),
                         )
                     }
                     message::Source::Server(_) => {
-                        let message = selectable_text(&message.text).style(theme::Text::Server);
+                        let message =
+                            selectable_text(&message.text).style(theme::selectable_text::info);
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
                     message::Source::Action => {
-                        let message = selectable_text(&message.text).style(theme::Text::Accent);
+                        let message =
+                            selectable_text(&message.text).style(theme::selectable_text::accent);
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
                     message::Source::Internal(message::source::Internal::Status(status)) => {
-                        let message =
-                            selectable_text(&message.text).style(theme::Text::Status(*status));
+                        let message = selectable_text(&message.text)
+                            .style(|theme| theme::selectable_text::status(theme, *status));
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
@@ -108,49 +118,58 @@ pub fn view<'a>(
     .width(Length::FillPortion(2))
     .height(Length::Fill);
 
+    let is_connected_to_channel = clients
+        .get_channels(&state.server)
+        .iter()
+        .any(|c| c == &state.channel);
     let users = clients.get_channel_users(&state.server, &state.channel);
     let channels = clients.get_channels(&state.server);
-    let nick_list = nick_list::view(users, config).map(Message::UserContext);
+    let nick_list = nick_list::view(users, &buffer, config).map(Message::UserContext);
 
     let show_text_input = match config.buffer.input_visibility {
-        data::buffer::InputVisibility::Focused => is_focused && status.connected(),
-        data::buffer::InputVisibility::Always => status.connected(),
+        data::buffer::InputVisibility::Focused => is_focused,
+        data::buffer::InputVisibility::Always => true,
     };
 
-    let text_input = show_text_input.then(|| {
-        column![
-            vertical_space(4),
-            input_view::view(
-                &state.input_view,
-                buffer,
-                users,
-                channels,
-                input_history,
-                is_focused
-            )
-            .map(Message::InputView)
-        ]
-        .width(Length::Fill)
+    // If topic toggles from None to Some then it messes with messages' scroll state,
+    // so produce a zero-height placeholder when topic is None.
+    let topic = topic(state, clients, users, settings, config).unwrap_or_else(|| column![].into());
+
+    let text_input = show_text_input.then(move || {
+        input_view::view(
+            &state.input_view,
+            buffer,
+            input,
+            users,
+            channels,
+            is_focused,
+            !is_connected_to_channel,
+        )
+        .map(Message::InputView)
     });
 
+    let content = column![topic, messages].spacing(4);
+
     let content = match (settings.users.visible, config.buffer.channel.users.position) {
         (true, data::channel::Position::Left) => {
-            row![nick_list, messages]
+            row![nick_list, content]
         }
         (true, data::channel::Position::Right) => {
-            row![messages, nick_list]
+            row![content, nick_list]
         }
-        (false, _) => { row![messages] }.height(Length::Fill),
+        (false, _) => { row![content] }.height(Length::Fill),
     };
 
-    let scrollable = column![container(content).height(Length::Fill)]
+    let body = column![]
+        .push(container(content).height(Length::Fill))
         .push_maybe(text_input)
+        .spacing(4)
         .height(Length::Fill);
 
-    container(scrollable)
+    container(body)
         .width(Length::Fill)
         .height(Length::Fill)
-        .padding(8)
+        .padding([4, 8, 8, 8])
         .into()
 }
 
@@ -158,7 +177,7 @@ pub fn view<'a>(
 pub struct Channel {
     pub server: Server,
     pub channel: String,
-    pub topic: Option<String>,
+
     pub scroll_view: scroll_view::State,
     pub input_view: input_view::State,
 }
@@ -168,7 +187,6 @@ impl Channel {
         Self {
             server,
             channel,
-            topic: None,
             scroll_view: scroll_view::State::new(),
             input_view: input_view::State::new(),
         }
@@ -226,9 +244,36 @@ impl Channel {
     }
 }
 
+fn topic<'a>(
+    state: &'a Channel,
+    clients: &'a data::client::Map,
+    users: &'a [User],
+    settings: &'a channel::Settings,
+    config: &'a Config,
+) -> Option<Element<'a, Message>> {
+    if !settings.topic.visible {
+        return None;
+    }
+
+    let topic = clients.get_channel_topic(&state.server, &state.channel)?;
+
+    Some(
+        topic::view(
+            topic.text.as_deref()?,
+            topic.who.as_deref(),
+            topic.time.as_ref(),
+            config.buffer.channel.topic.max_lines,
+            users,
+            &state.buffer(),
+            config,
+        )
+        .map(Message::UserContext),
+    )
+}
+
 mod nick_list {
-    use data::{Config, User};
-    use iced::widget::{column, container, scrollable, text};
+    use data::{Buffer, Config, User};
+    use iced::widget::{column, container, scrollable, text, Scrollable};
     use iced::Length;
     use user_context::Message;
 
@@ -236,30 +281,32 @@ mod nick_list {
     use crate::theme;
     use crate::widget::Element;
 
-    pub fn view<'a>(users: &[User], config: &'a Config) -> Element<'a, Message> {
+    pub fn view<'a>(
+        users: &'a [User],
+        buffer: &Buffer,
+        config: &'a Config,
+    ) -> Element<'a, Message> {
         let column = column(users.iter().map(|user| {
-            let content = text(format!(
-                "{}{}",
-                user.highest_access_level(),
-                user.nickname()
-            ))
-            .style(theme::Text::Nickname(
-                user.color_seed(&config.buffer.channel.users.color),
-                user.is_away(),
-            ));
+            let content = text(user).style(|theme| {
+                theme::text::nickname(
+                    theme,
+                    user.color_seed(&config.buffer.channel.users.color),
+                    user.is_away(),
+                )
+            });
 
-            user_context::view(content, user.clone())
+            user_context::view(content, user, buffer.clone())
         }))
         .padding(4)
         .spacing(1);
 
-        container(
-            scrollable(column)
-                .direction(scrollable::Direction::Vertical(
-                    scrollable::Properties::new().width(1).scroller_width(1),
-                ))
-                .style(theme::Scrollable::Hidden),
-        )
+        container(Scrollable::with_direction_and_style(
+            column,
+            scrollable::Direction::Vertical(
+                scrollable::Properties::new().width(1).scroller_width(1),
+            ),
+            theme::scrollable::hidden,
+        ))
         .width(Length::Shrink)
         .max_width(120)
         .height(Length::Fill)
