@@ -1,4 +1,5 @@
-use data::user::User;
+use data::input::{Cache, Draft};
+use data::user::{Nick, User};
 use data::{client, history, Buffer, Input};
 use iced::Command;
 
@@ -10,27 +11,29 @@ pub enum Event {
 
 #[derive(Debug, Clone)]
 pub enum Message {
-    Input(String),
+    Input(Draft),
     Send(Input),
-    Completion(String),
+    Completion(Draft),
 }
 
 pub fn view<'a>(
     state: &'a State,
     buffer: Buffer,
+    cache: Cache<'a>,
     users: &'a [User],
     channels: &'a [String],
-    history: &'a [String],
     buffer_focused: bool,
+    disabled: bool,
 ) -> Element<'a, Message> {
     input(
         state.input_id.clone(),
         buffer,
-        &state.input,
+        cache.draft,
+        cache.history,
         users,
         channels,
-        history,
         buffer_focused,
+        disabled,
         Message::Input,
         Message::Send,
         Message::Completion,
@@ -40,7 +43,6 @@ pub fn view<'a>(
 #[derive(Debug, Clone)]
 pub struct State {
     input_id: input::Id,
-    input: String,
 }
 
 impl Default for State {
@@ -53,7 +55,6 @@ impl State {
     pub fn new() -> Self {
         Self {
             input_id: input::Id::unique(),
-            input: String::default(),
         }
     }
 
@@ -64,26 +65,35 @@ impl State {
         history: &mut history::Manager,
     ) -> (Command<Message>, Option<Event>) {
         match message {
-            Message::Input(input) => {
-                self.input = input;
+            Message::Input(draft) => {
+                history.record_draft(draft);
 
                 (Command::none(), None)
             }
             Message::Send(input) => {
-                self.input.clear();
-
                 if let Some(encoded) = input.encoded() {
                     clients.send(input.buffer(), encoded);
                 }
 
                 if let Some(nick) = clients.nickname(input.server()) {
-                    history.record_input(input, nick);
+                    let mut user = nick.to_owned().into();
+
+                    // Resolve our attributes if sending this message in a channel
+                    if let Buffer::Channel(server, channel) = input.buffer() {
+                        if let Some(user_with_attributes) =
+                            clients.resolve_user_attributes(server, channel, &user)
+                        {
+                            user = user_with_attributes.clone();
+                        }
+                    }
+
+                    history.record_input(input, user);
                 }
 
                 (Command::none(), Some(Event::InputSent))
             }
-            Message::Completion(input) => {
-                self.input = input;
+            Message::Completion(draft) => {
+                history.record_draft(draft);
 
                 (input::move_cursor_to_end(self.input_id.clone()), None)
             }
@@ -98,15 +108,24 @@ impl State {
         input::reset(self.input_id.clone())
     }
 
-    pub fn insert_user(&mut self, user: User) -> Command<Message> {
-        if self.input.is_empty() {
-            self.input = format!("{}: ", user.nickname());
-        } else if self.input.ends_with(' ') {
-            self.input = format!("{}{}", self.input, user.nickname());
+    pub fn insert_user(
+        &mut self,
+        nick: Nick,
+        buffer: Buffer,
+        history: &mut history::Manager,
+    ) -> Command<Message> {
+        let mut text = history.input(&buffer).draft.to_string();
+
+        if text.is_empty() {
+            text = format!("{}: ", nick);
+        } else if text.ends_with(' ') {
+            text = format!("{}{}", text, nick);
         } else {
-            self.input = format!("{} {}", self.input, user.nickname());
+            text = format!("{} {}", text, nick);
         }
 
+        history.record_draft(Draft { buffer, text });
+
         input::move_cursor_to_end(self.input_id.clone())
     }
 }
