@@ -1,11 +1,11 @@
 use data::user::Nick;
-use data::{client, history, message, Config, Server};
+use data::{history, message, Config, Server};
 use iced::widget::{column, container, row, vertical_space};
 use iced::{Command, Length};
 
 use super::{input_view, scroll_view, user_context};
 use crate::theme;
-use crate::widget::{selectable_text, Collection, Element};
+use crate::widget::{selectable_text, Element};
 
 #[derive(Debug, Clone)]
 pub enum Message {
@@ -20,14 +20,14 @@ pub enum Event {
 
 pub fn view<'a>(
     state: &'a Query,
-    status: client::Status,
     clients: &'a data::client::Map,
     history: &'a history::Manager,
     config: &'a Config,
     is_focused: bool,
 ) -> Element<'a, Message> {
+    let status = clients.status(&state.server);
     let buffer = state.buffer();
-    let input_history = history.input_history(&buffer);
+    let input = history.input(&buffer);
 
     let messages = container(
         scroll_view::view(
@@ -36,21 +36,28 @@ pub fn view<'a>(
             history,
             config,
             |message| {
-                let timestamp = config
-                    .buffer
-                    .format_timestamp(&message.server_time)
-                    .map(|timestamp| selectable_text(timestamp).style(theme::Text::Transparent));
+                let timestamp =
+                    config
+                        .buffer
+                        .format_timestamp(&message.server_time)
+                        .map(|timestamp| {
+                            selectable_text(timestamp).style(theme::selectable_text::transparent)
+                        });
 
                 match message.target.source() {
                     message::Source::User(user) => {
                         let nick = user_context::view(
                             selectable_text(config.buffer.nickname.brackets.format(user)).style(
-                                theme::Text::Nickname(
-                                    user.color_seed(&config.buffer.nickname.color),
-                                    false,
-                                ),
+                                |theme| {
+                                    theme::selectable_text::nickname(
+                                        theme,
+                                        user.color_seed(&config.buffer.nickname.color),
+                                        false,
+                                    )
+                                },
                             ),
-                            user.clone(),
+                            user,
+                            state.buffer(),
                         )
                         .map(scroll_view::Message::UserContext);
 
@@ -69,18 +76,20 @@ pub fn view<'a>(
                         )
                     }
                     message::Source::Server(_) => {
-                        let message = selectable_text(&message.text).style(theme::Text::Server);
+                        let message =
+                            selectable_text(&message.text).style(theme::selectable_text::info);
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
                     message::Source::Action => {
-                        let message = selectable_text(&message.text).style(theme::Text::Accent);
+                        let message =
+                            selectable_text(&message.text).style(theme::selectable_text::accent);
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
                     message::Source::Internal(message::source::Internal::Status(status)) => {
-                        let message =
-                            selectable_text(&message.text).style(theme::Text::Status(*status));
+                        let message = selectable_text(&message.text)
+                            .style(|theme| theme::selectable_text::status(theme, *status));
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
@@ -92,22 +101,23 @@ pub fn view<'a>(
     .height(Length::Fill);
 
     let show_text_input = match config.buffer.input_visibility {
-        data::buffer::InputVisibility::Focused => is_focused && status.connected(),
-        data::buffer::InputVisibility::Always => status.connected(),
+        data::buffer::InputVisibility::Focused => is_focused,
+        data::buffer::InputVisibility::Always => true,
     };
 
     let channels = clients.get_channels(&state.server);
 
     let text_input = show_text_input.then(|| {
         column![
-            vertical_space(4),
+            vertical_space().height(4),
             input_view::view(
                 &state.input_view,
                 buffer,
+                input,
                 &[],
                 channels,
-                input_history,
-                is_focused
+                is_focused,
+                !status.connected()
             )
             .map(Message::InputView)
         ]
