@@ -2,7 +2,7 @@ use data::message::Limit;
 use data::server::Server;
 use data::user::Nick;
 use data::{history, time, Config};
-use iced::widget::{column, container, horizontal_rule, scrollable};
+use iced::widget::{column, container, horizontal_rule, row, text, scrollable, Scrollable};
 use iced::{Command, Length};
 
 use super::user_context;
@@ -46,20 +46,14 @@ pub fn view<'a>(
         new_messages,
     }) = (match kind {
         Kind::Server(server) => {
-            history.get_server_messages(server, Some(state.limit), &config.buffer.server_messages)
+            history.get_server_messages(server, Some(state.limit), &config.buffer)
+        }
+        Kind::Channel(server, channel) => {
+            history.get_channel_messages(server, channel, Some(state.limit), &config.buffer)
+        }
+        Kind::Query(server, user) => {
+            history.get_query_messages(server, user, Some(state.limit), &config.buffer)
         }
-        Kind::Channel(server, channel) => history.get_channel_messages(
-            server,
-            channel,
-            Some(state.limit),
-            &config.buffer.server_messages,
-        ),
-        Kind::Query(server, user) => history.get_query_messages(
-            server,
-            user,
-            Some(state.limit),
-            &config.buffer.server_messages,
-        ),
     })
     else {
         return column![].into();
@@ -87,30 +81,45 @@ pub fn view<'a>(
     let show_divider = !new.is_empty() || matches!(status, Status::Idle(Anchor::Bottom));
 
     let content = if show_divider {
-        let divider = container(horizontal_rule(1).style(theme::Rule::Unread))
-            .width(Length::Fill)
-            .padding(5);
+        let font_size = config.font.size.map(f32::from).unwrap_or(theme::TEXT_SIZE) - 1.0;
+
+        let divider = row![
+            container(horizontal_rule(1))
+                .width(Length::Fill)
+                .padding([0, 6, 0, 0]),
+            text("backlog")
+                .size(font_size)
+                .style(theme::text::transparent),
+            container(horizontal_rule(1))
+                .width(Length::Fill)
+                .padding([0, 0, 0, 6])
+        ]
+        .padding(2)
+        .align_items(iced::Alignment::Center);
+
         column![column(old), divider, column(new)]
     } else {
         column![column(old), column(new)]
     };
 
-    scrollable(container(content).width(Length::Fill).padding([0, 8]))
-        .direction(scrollable::Direction::Vertical(
+    Scrollable::with_direction(
+        container(content).width(Length::Fill).padding([0, 8]),
+        scrollable::Direction::Vertical(
             scrollable::Properties::default()
                 .alignment(status.alignment())
                 .width(5)
                 .scroller_width(5),
-        ))
-        .on_scroll(move |viewport| Message::Scrolled {
-            count,
-            remaining,
-            oldest,
-            status,
-            viewport,
-        })
-        .id(state.scrollable.clone())
-        .into()
+        ),
+    )
+    .on_scroll(move |viewport| Message::Scrolled {
+        count,
+        remaining,
+        oldest,
+        status,
+        viewport,
+    })
+    .id(state.scrollable.clone())
+    .into()
 }
 
 #[derive(Debug, Clone)]
