@@ -1,10 +1,10 @@
-use data::{client, history, message, Config};
+use data::{history, message, Config};
 use iced::widget::{column, container, row, vertical_space};
 use iced::{Command, Length};
 
 use super::{input_view, scroll_view};
 use crate::theme;
-use crate::widget::{selectable_text, Collection, Element};
+use crate::widget::{selectable_text, Element};
 
 #[derive(Debug, Clone)]
 pub enum Message {
@@ -14,14 +14,14 @@ pub enum Message {
 
 pub fn view<'a>(
     state: &'a Server,
-    status: client::Status,
     clients: &'a data::client::Map,
     history: &'a history::Manager,
     config: &'a Config,
     is_focused: bool,
 ) -> Element<'a, Message> {
+    let status = clients.status(&state.server);
     let buffer = state.buffer();
-    let input_history = history.input_history(&buffer);
+    let input = history.input(&buffer);
 
     let messages = container(
         scroll_view::view(
@@ -30,20 +30,24 @@ pub fn view<'a>(
             history,
             config,
             |message| {
-                let timestamp = config
-                    .buffer
-                    .format_timestamp(&message.server_time)
-                    .map(|timestamp| selectable_text(timestamp).style(theme::Text::Transparent));
+                let timestamp =
+                    config
+                        .buffer
+                        .format_timestamp(&message.server_time)
+                        .map(|timestamp| {
+                            selectable_text(timestamp).style(theme::selectable_text::transparent)
+                        });
 
                 match message.target.source() {
                     message::Source::Server(_) => {
-                        let message = selectable_text(&message.text).style(theme::Text::Server);
+                        let message =
+                            selectable_text(&message.text).style(theme::selectable_text::info);
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
                     message::Source::Internal(message::source::Internal::Status(status)) => {
-                        let message =
-                            selectable_text(&message.text).style(theme::Text::Status(*status));
+                        let message = selectable_text(&message.text)
+                            .style(|theme| theme::selectable_text::status(theme, *status));
 
                         Some(container(row![].push_maybe(timestamp).push(message)).into())
                     }
@@ -56,22 +60,23 @@ pub fn view<'a>(
     .height(Length::Fill);
 
     let show_text_input = match config.buffer.input_visibility {
-        data::buffer::InputVisibility::Focused => is_focused && status.connected(),
-        data::buffer::InputVisibility::Always => status.connected(),
+        data::buffer::InputVisibility::Focused => is_focused,
+        data::buffer::InputVisibility::Always => true,
     };
 
     let channels = clients.get_channels(&state.server);
 
     let text_input = show_text_input.then(|| {
         column![
-            vertical_space(4),
+            vertical_space().height(4),
             input_view::view(
                 &state.input_view,
                 buffer,
+                input,
                 &[],
                 channels,
-                input_history,
-                is_focused
+                is_focused,
+                !status.connected()
             )
             .map(Message::InputView)
         ]
@@ -93,7 +98,7 @@ pub fn view<'a>(
 pub struct Server {
     pub server: data::server::Server,
     pub scroll_view: scroll_view::State,
-    input_view: input_view::State,
+    pub input_view: input_view::State,
 }
 
 impl Server {
