@@ -38,9 +38,13 @@ pub fn people<'a>() -> Text<'a> {
     to_text('\u{f4db}')
 }
 
+pub fn topic<'a>() -> Text<'a> {
+    to_text('\u{f5af}')
+}
+
 fn to_text<'a>(unicode: char) -> Text<'a> {
     text(unicode.to_string())
-        .style(theme::Text::Primary)
+        .style(theme::text::primary)
         .line_height(LineHeight::Relative(1.0))
         .size(theme::ICON_SIZE)
         .font(font::ICON)
