@@ -17,7 +17,7 @@ use std::env;
 use std::time::{Duration, Instant};
 
 use data::config::{self, Config};
-use data::{environment, server};
+use data::{environment, server, User};
 use iced::widget::container;
 use iced::{executor, Application, Command, Length, Subscription};
 use screen::{dashboard, help, welcome};
@@ -319,18 +319,31 @@ impl Application for Halloy {
 
                     messages.into_iter().for_each(|message| {
                         for event in self.clients.receive(&server, message) {
+                            // Resolve a user using client state which stores attributes
+                            let resolve_user_attributes = |user: &User, channel: &str| {
+                                self.clients
+                                    .resolve_user_attributes(&server, channel, user)
+                                    .cloned()
+                            };
+
                             match event {
                                 data::client::Event::Single(encoded, our_nick) => {
-                                    if let Some(message) =
-                                        data::Message::received(encoded, our_nick, &self.config)
-                                    {
+                                    if let Some(message) = data::Message::received(
+                                        encoded,
+                                        our_nick,
+                                        &self.config,
+                                        resolve_user_attributes,
+                                    ) {
                                         dashboard.record_message(&server, message);
                                     }
                                 }
                                 data::client::Event::WithTarget(encoded, our_nick, target) => {
-                                    if let Some(message) =
-                                        data::Message::received(encoded, our_nick, &self.config)
-                                    {
+                                    if let Some(message) = data::Message::received(
+                                        encoded,
+                                        our_nick,
+                                        &self.config,
+                                        resolve_user_attributes,
+                                    ) {
                                         dashboard
                                             .record_message(&server, message.with_target(target));
                                     }
@@ -387,9 +400,12 @@ impl Application for Halloy {
                                     our_nick,
                                     notification,
                                 ) => {
-                                    if let Some(message) =
-                                        data::Message::received(encoded, our_nick, &self.config)
-                                    {
+                                    if let Some(message) = data::Message::received(
+                                        encoded,
+                                        our_nick,
+                                        &self.config,
+                                        resolve_user_attributes,
+                                    ) {
                                         dashboard.record_message(&server, message);
                                     }
 
@@ -456,7 +472,7 @@ impl Application for Halloy {
         container(content)
             .width(Length::Fill)
             .height(Length::Fill)
-            .style(theme::Container::Primary)
+            .style(theme::container::primary)
             .into()
     }
 
