@@ -15,7 +15,7 @@ use self::command_bar::CommandBar;
 use self::pane::Pane;
 use self::side_menu::SideMenu;
 use crate::buffer::{self, Buffer};
-use crate::widget::{anchored_overlay, selectable_text, shortcut, Collection, Element};
+use crate::widget::{anchored_overlay, selectable_text, shortcut, Element};
 use crate::{event, theme, Theme};
 
 const SAVE_AFTER: Duration = Duration::from_secs(3);
@@ -110,40 +110,71 @@ impl Dashboard {
 
                         if let Some(buffer::Event::UserContext(event)) = event {
                             match event {
-                                buffer::user_context::Event::SendWhois(user) => {
-                                    if let Some(buffer) = pane.buffer.data() {
-                                        let command =
-                                            data::Command::Whois(None, user.nickname().to_string());
+                                buffer::user_context::Event::ToggleAccessLevel(nick, mode) => {
+                                    let Some(buffer) = pane.buffer.data() else {
+                                        return Command::none();
+                                    };
 
-                                        let input = data::Input::command(buffer, command);
+                                    let Some(target) = buffer.target() else {
+                                        return Command::none();
+                                    };
+
+                                    let command = data::Command::Mode(
+                                        target,
+                                        Some(mode),
+                                        vec![nick.to_string()],
+                                    );
+                                    let input = data::Input::command(buffer.clone(), command);
+
+                                    if let Some(encoded) = input.encoded() {
+                                        clients.send(input.buffer(), encoded);
+                                    }
+                                }
+                                buffer::user_context::Event::SendWhois(nick) => {
+                                    if let Some(buffer) = pane.buffer.data() {
+                                        let command = data::Command::Whois(None, nick.to_string());
+
+                                        let input = data::Input::command(buffer.clone(), command);
 
                                         if let Some(encoded) = input.encoded() {
                                             clients.send(input.buffer(), encoded);
                                         }
 
-                                        if let Some(message) = clients
-                                            .nickname(input.server())
-                                            .and_then(|nick| input.message(nick))
-                                        {
-                                            self.history.record_message(input.server(), message);
+                                        if let Some(nick) = clients.nickname(buffer.server()) {
+                                            let mut user = nick.to_owned().into();
+
+                                            // Resolve our attributes if sending this message in a channel
+                                            if let data::Buffer::Channel(server, channel) = &buffer
+                                            {
+                                                if let Some(user_with_attributes) = clients
+                                                    .resolve_user_attributes(server, channel, &user)
+                                                {
+                                                    user = user_with_attributes.clone();
+                                                }
+                                            }
+
+                                            if let Some(message) = input.message(user) {
+                                                self.history
+                                                    .record_message(input.server(), message);
+                                            }
                                         }
                                     }
                                 }
-                                buffer::user_context::Event::OpenQuery(user) => {
+                                buffer::user_context::Event::OpenQuery(nick) => {
                                     if let Some(data) = pane.buffer.data() {
-                                        let buffer = data::Buffer::Query(
-                                            data.server().clone(),
-                                            user.nickname().to_owned(),
-                                        );
+                                        let buffer =
+                                            data::Buffer::Query(data.server().clone(), nick);
                                         return self.open_buffer(buffer, config);
                                     }
                                 }
-                                buffer::user_context::Event::SingleClick(user) => {
-                                    let Some((_, pane)) = self.get_focused_mut() else {
+                                buffer::user_context::Event::SingleClick(nick) => {
+                                    let Some((_, pane, history)) =
+                                        self.get_focused_with_history_mut()
+                                    else {
                                         return Command::none();
                                     };
 
-                                    return pane.buffer.insert_user_to_input(user).map(
+                                    return pane.buffer.insert_user_to_input(nick, history).map(
                                         move |message| {
                                             Message::Pane(pane::Message::Buffer(id, message))
                                         },
@@ -162,6 +193,12 @@ impl Dashboard {
                         self.last_changed = Some(Instant::now());
                     }
                 }
+                pane::Message::ToggleShowTopic => {
+                    if let Some((_, pane)) = self.get_focused_mut() {
+                        pane.update_settings(|settings| settings.channel.topic.toggle_visibility());
+                        self.last_changed = Some(Instant::now());
+                    }
+                }
                 pane::Message::MaximizePane => self.maximize_pane(),
             },
             Message::SideMenu(message) => {
@@ -530,7 +567,7 @@ impl Dashboard {
                 container(Space::new(Length::Fill, Length::Fill))
                     .width(Length::Fill)
                     .height(Length::Fill)
-                    .style(theme::Container::SemiTransparent),
+                    .style(theme::container::semi_transparent),
                 anchored_overlay::Anchor::BelowTopCentered,
                 0.0,
             );
@@ -777,6 +814,15 @@ impl Dashboard {
         self.panes.get_mut(pane).map(|state| (pane, state))
     }
 
+    fn get_focused_with_history_mut(
+        &mut self,
+    ) -> Option<(pane_grid::Pane, &mut Pane, &mut history::Manager)> {
+        let pane = self.focus?;
+        self.panes
+            .get_mut(pane)
+            .map(|state| (pane, state, &mut self.history))
+    }
+
     fn focus_pane(&mut self, pane: pane_grid::Pane) -> Command<Message> {
         if self.focus != Some(pane) {
             self.focus = Some(pane);
