@@ -17,13 +17,14 @@ pub fn input<'a, Message>(
     id: Id,
     buffer: Buffer,
     input: &'a str,
+    history: &'a [String],
     users: &'a [User],
     channels: &'a [String],
-    history: &'a [String],
     buffer_focused: bool,
-    on_input: impl Fn(String) -> Message + 'a,
+    disabled: bool,
+    on_input: impl Fn(input::Draft) -> Message + 'a,
     on_submit: impl Fn(data::Input) -> Message + 'a,
-    on_completion: impl Fn(String) -> Message + 'a,
+    on_completion: impl Fn(input::Draft) -> Message + 'a,
 ) -> Element<'a, Message>
 where
     Message: 'a + Clone,
@@ -36,6 +37,7 @@ where
         channels,
         history,
         buffer_focused,
+        disabled,
         on_input: Box::new(on_input),
         on_submit: Box::new(on_submit),
         on_completion: Box::new(on_completion),
@@ -66,9 +68,10 @@ pub struct Input<'a, Message> {
     channels: &'a [String],
     history: &'a [String],
     buffer_focused: bool,
-    on_input: Box<dyn Fn(String) -> Message + 'a>,
+    disabled: bool,
+    on_input: Box<dyn Fn(data::input::Draft) -> Message + 'a>,
     on_submit: Box<dyn Fn(data::Input) -> Message + 'a>,
-    on_completion: Box<dyn Fn(String) -> Message + 'a>,
+    on_completion: Box<dyn Fn(data::input::Draft) -> Message + 'a>,
 }
 
 #[derive(Default)]
@@ -95,7 +98,10 @@ where
 
                 state.completion.process(&input, self.users, self.channels);
 
-                Some((self.on_input)(input))
+                Some((self.on_input)(input::Draft {
+                    buffer: self.buffer.clone(),
+                    text: input,
+                }))
             }
             Event::Send => {
                 // Reset error state
@@ -105,7 +111,11 @@ where
 
                 if let Some(entry) = state.completion.select() {
                     let new_input = entry.complete_input(self.input);
-                    Some((self.on_completion)(new_input))
+
+                    Some((self.on_completion)(input::Draft {
+                        buffer: self.buffer.clone(),
+                        text: new_input,
+                    }))
                 } else if !self.input.is_empty() {
                     state.completion.reset();
 
@@ -126,7 +136,11 @@ where
             Event::Tab => {
                 if let Some(entry) = state.completion.tab() {
                     let new_input = entry.complete_input(self.input);
-                    Some((self.on_completion)(new_input))
+
+                    Some((self.on_completion)(input::Draft {
+                        buffer: self.buffer.clone(),
+                        text: new_input,
+                    }))
                 } else {
                     None
                 }
@@ -150,7 +164,10 @@ where
                         .completion
                         .process(&new_input, self.users, self.channels);
 
-                    return Some((self.on_completion)(new_input));
+                    return Some((self.on_completion)(input::Draft {
+                        buffer: self.buffer.clone(),
+                        text: new_input,
+                    }));
                 }
 
                 None
@@ -171,7 +188,10 @@ where
                         new_input
                     };
 
-                    return Some((self.on_completion)(new_input));
+                    return Some((self.on_completion)(input::Draft {
+                        buffer: self.buffer.clone(),
+                        text: new_input,
+                    }));
                 }
 
                 None
@@ -181,18 +201,21 @@ where
 
     fn view(&self, state: &Self::State) -> Element<'_, Self::Event> {
         let style = if state.error.is_some() {
-            theme::TextInput::Error
+            theme::text_input::error
         } else {
-            theme::TextInput::Default
+            theme::text_input::primary
         };
 
-        let text_input = text_input("Send message...", self.input)
-            .on_input(Event::Input)
+        let mut text_input = text_input("Send message...", self.input)
             .on_submit(Event::Send)
             .id(self.id.clone())
             .padding(8)
             .style(style);
 
+        if !self.disabled {
+            text_input = text_input.on_input(Event::Input);
+        }
+
         // Add tab support
         let mut input = key_press(
             text_input,
@@ -232,10 +255,10 @@ where
 }
 
 fn error<'a, Message: 'a>(error: impl ToString) -> Element<'a, Message> {
-    container(text(error).style(theme::Text::Error))
+    container(text(error).style(theme::text::error))
         .center_y()
         .padding(8)
-        .style(theme::Container::Primary)
+        .style(theme::container::context)
         .into()
 }
 
